﻿
namespace Parade.Metadata.Asf
{
	/// <summary>
	/// タグ情報を表します。
	/// </summary>
	class TagData
	{
		/// <summary>
		/// インスタンスを初期化します。
		/// </summary>
		/// <param name="value">タグの内容。</param>
		public TagData( object value )
		{
			this._value = value;
		}

		/// <summary>
		/// インスタンスを初期化します。
		/// </summary>
		/// <param name="position">ストーリム上の読み出し位置 ( バイト単位 )。</param>
		/// <param name="length">ストリームから読み取るサイズ ( バイト単位 )。</param>
		/// <param name="type">データ型。</param>
		/// <param name="innerType">ファイル内のデータ型。</param>
		/// <param name="type">タグ情報を読み込むためのストリーム。</param>
		public TagData( long position, int length, TagDataType type )
		{
			this._value = new ReadInfo( position, length, type );
		}

		/// <summary>
		/// タグ情報の編集がおこなわれたことを示す値を取得します。
		/// </summary>
		public bool IsEdited { get; private set; }

		/// <summary>
		/// タグの内容を取得または設定します。
		/// </summary>
		public object Value
		{
			get
			{
				return this._value;
			}
			set
			{
				this._value   = value;
				this.IsEdited = true;
			}
		}

		/// <summary>
		/// タグの内容。遅延読み込みをおこなう場合は ReadInfo を格納する。
		/// </summary>
		private object _value;

		/// <summary>
		/// 遅延読み込みをおこなうための情報を格納します。
		/// </summary>
		public class ReadInfo
		{
			/// <summary>
			/// インスタンスを初期化します。
			/// </summary>
			/// <param name="position">ストーリム上の読み出し位置 ( バイト単位 )。</param>
			/// <param name="length">ストリームから読み取るサイズ ( バイト単位 )。</param>
			/// <param name="type">データ型。</param>
			public ReadInfo( long position, int length, TagDataType type )
			{
				this.Length    = length;
				this.Position  = position;
				this.Type      = type;
			}
			
			/// <summary>
			/// ストリームから読み取るサイズ ( バイト単位 ) を取得します。
			/// </summary>
			public int Length { get; private set; }

			/// <summary>
			/// ストーリム上の読み出し位置 ( バイト単位 ) を取得します。
			/// </summary>
			public long Position { get; private set; }

			/// <summary>
			/// データ型を取得します。
			/// </summary>
			public TagDataType Type { get; private set; }
		}
	}
}
